/**
* @file Uzewav.h
*
* @brief Uzewav Global defines and build options
*
* @version 1.1
*
* @date 2012 
*
* @author  Hartmut Wendt  info@hwhardsoft.de.vu	<http://www.hwhardsoft.de.vu>
*
* @copyright GNU Public License V3
* 
* @note
*   This program is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*   You should have received a copy of the GNU General Public License
*   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __UZEWAV_H_
	#define __UZEWAV_H_

extern void mmc_processMixer();
extern void mmc_mixerStart(uint32_t lba);
extern void mmc_mixerStop();
extern uint32_t mmc_mixerGetCurrentSector();


#define FAT_ATTR_READONLY	0x01
#define FAT_ATTR_HIDDEN		0x02
#define FAT_ATTR_SYSTEM		0x04
#define FAT_ATTR_VOLUME		0x08
#define FAT_ATTR_DIRECTORY	0x10
#define FAT_ATTR_ARCHIVE	0x20
#define FAT_ATTR_DEVICE		0x40

/** 
 @brief count of bytes per sector - fixed for regular SD
*/
#define BYTES_PER_SECTOR 512 	//fixed for regular SD

/** 
 @brief maximal count of voice files per language
*/
#define MAX_WAVES	32

typedef struct{
	//header
	unsigned char chunkID[4];
	unsigned long chunkSize;
	unsigned char format[4];
	
	//fmt chunk
	unsigned char subchunk1ID[4];
	unsigned long subchunk1Size;
	unsigned int audioFormat;
	unsigned int numChannels;
	unsigned long sampleRate;
	unsigned long byteRate;
	unsigned int blockAlign;
	unsigned int bitsPerSample;

	//data chunk
	unsigned char subchunk2ID[4];
	unsigned long subchunk2Size;

} RIFFheader;

typedef struct{
	unsigned char filename[8]; //zero padded
	unsigned char extension[3];//
	unsigned char fileAttributes;
	unsigned char reserved;
	unsigned char creationTimeMillis;
	unsigned int creationTime;
	unsigned int creationDate;
	unsigned int lastAccessDate;
	unsigned int eaIndex;
	unsigned int lastModifiedTime;
	unsigned int lastModifiedDate;
	unsigned int firstCluster;
	unsigned long fileSize;

} DirectoryTableEntry;


typedef struct{
	unsigned char state;
	unsigned char startHead;
	unsigned int startCylinder;
	unsigned char type;
	unsigned char endHead;
	unsigned int endCylinder;
	unsigned long startSector; //boot record starts at this sector
	unsigned long size; //in sectors

} PartitionEntry;


typedef struct {
	unsigned char execCode[446];
	PartitionEntry partition1;
	PartitionEntry partition2;
	PartitionEntry partition3;
	PartitionEntry partition4;
	int marker;
} MBR;

typedef struct {
	unsigned char jmp[3];
	unsigned char oemName[8];
	unsigned int bytesPerSector;
	unsigned char sectorsPerCluster;
	unsigned int reservedSectors;
	unsigned char fatCopies;
	unsigned int maxRootDirectoryEntries;
	unsigned int totalSectorsLegacy;
	unsigned char mediaDescriptor;
	unsigned int sectorsPerFat;
	unsigned int sectorPerTrack;
	unsigned int numbersOfHeads;
	unsigned long hiddenSectors;
	unsigned long totalSectors;
	unsigned char physicalDriveNumber;
	unsigned char reserved;
	unsigned char extendedBootSignature;
	unsigned long serialNumber;
	unsigned char volumeLabel[11];	
	unsigned char bootCode[448];
	unsigned int signature;

} BootRecord;

union SectorData {
	unsigned char buffer[BYTES_PER_SECTOR];
	BootRecord bootRecord;
	MBR mbr;
	DirectoryTableEntry files[16];
	RIFFheader riffHeader;
} sector;


// program modes
/** @brief found files
*
* - first sector of file
* - file size 
*/
typedef struct{
	unsigned long firstSector;
	unsigned long fileSize;	
} File;


u8 SD_init(void);
int convert_filename(unsigned char *fname);
long GetFileSector(DirectoryTableEntry *file);

void fat_init(unsigned char *buffer);
u8 find_files(const char *s);

void startSong();
void stopSong();
void stop_auto();
void play_queue();



#define SPR_SONG_CUR 0
#define SPR_MOUSE 2


// german voices
#define ger_0			0
#define ger_1			1
#define ger_2			2
#define ger_3			3
#define ger_4			4
#define ger_5			5
#define ger_6			6
#define ger_7			7
#define ger_8			8
#define ger_9			9
#define ger_10			10
#define ger_11			11
#define ger_12			12
#define ger_14			13
#define ger_16			14
#define ger_17			15
#define ger_20			16
#define ger_30			17
#define ger_40			18
#define ger_50			19
#define ger_60			20
#define ger_70			21
#define ger_80			22
#define ger_90			23
#define ger_100			24
#define ger_minus		25
#define ger_grad		26
#define ger_celsius		27
#define ger_fahrenheit	28
#define ger_und			29


// english voices
#define eng_0			0
#define eng_1			1
#define eng_2			2
#define eng_3			3
#define eng_4			4
#define eng_5			5
#define eng_6			6
#define eng_7			7
#define eng_8			8
#define eng_9			9
#define eng_10			10
#define eng_11			11
#define eng_12			12
#define eng_13			13
#define eng_15			14
#define eng_teen		15
#define eng_20			16
#define eng_30			17
#define eng_40			18
#define eng_50			19
#define eng_60			20
#define eng_70			21
#define eng_80			22
#define eng_90			23
#define eng_100			24
#define eng_minus		25
#define eng_degree		26
#define eng_celsius		27
#define eng_fahrenheit	28




#endif
